"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var chatty = require("@looker/chatty");

function createElement(name) {
  let props = arguments.length > 1 && void 0 !== arguments[1] ? arguments[1] : {}, children = arguments.length > 2 && void 0 !== arguments[2] ? arguments[2] : [];
  const element = document.createElement(name);
  for (const [name, value] of Object.entries(props)) "style" === name ? Object.assign(element.style, props.style) : element.setAttribute(name, value);
  for (const child of Array.isArray(children) ? children : [ children ]) element.append(child);
  return element;
}

const getChartUrl = options => {
  try {
    const url = new URL(options.baseUrl);
    return url.pathname = [ url.pathname, "/" === url.pathname.slice(-1) ? "" : "/", "embed/charts" ].join(""), 
    url.search = "id=".concat(options.chartId, "&sdk=1"), url.search += options.autorefresh ? "&autorefresh=".concat(options.autorefresh) : "", 
    url.search += options.filter ? "&filter=".concat(encodeURIComponent(JSON.stringify(options.filter))) : "", 
    url.toString();
  } catch (e) {
    throw new Error("Base URL must be a valid URL");
  }
}, parseCSSMeasurement = value => "string" == typeof value ? value : "number" == typeof value ? "".concat(value, "px") : null;

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter((function(sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    }))), keys.push.apply(keys, symbols);
  }
  return keys;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach((function(key) {
      _defineProperty(target, key, source[key]);
    })) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach((function(key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    }));
  }
  return target;
}

function _defineProperty(obj, key, value) {
  return key in obj ? Object.defineProperty(obj, key, {
    value: value,
    enumerable: !0,
    configurable: !0,
    writable: !0
  }) : obj[key] = value, obj;
}

class EmbeddedChart {
  constructor(element, iframe, connection, options) {
    _defineProperty(this, "options", void 0), _defineProperty(this, "connection", void 0), 
    _defineProperty(this, "iframe", void 0), _defineProperty(this, "element", void 0), 
    this.element = element, this.iframe = iframe, this.connection = connection, this.options = _objectSpread({}, options);
  }
  async refresh() {
    return await this.connection.sendAndReceive("refresh");
  }
  async set(name, value) {
    return "autorefresh" === name ? "number" == typeof value ? await this.connection.sendAndReceive("set", name, value) : Promise.reject("autorefresh property value should be a number") : Promise.reject("Unknown property name");
  }
  async get(name) {
    if ("autorefresh" === name) {
      const [result] = await this.connection.sendAndReceive("get", name);
      return result;
    }
    return Promise.reject("Unknown property name");
  }
}

const getChartOptions = options => {
  if ("object" != typeof options || null === options) throw new Error("Options argument must be an object");
  const {baseUrl: baseUrl, chartId: chartId, filter: filter, autorefresh: autorefresh, width: width, height: height} = options;
  if ("string" != typeof baseUrl || 0 === baseUrl.length) throw new Error("Base URL must be a valid URL");
  if ("string" != typeof chartId || 0 === chartId.length) throw new Error("Chart ID must be specified");
  if (void 0 !== width && ![ "number", "string" ].includes(typeof width)) throw new Error("Width must be a string or number if specified");
  if (void 0 !== height && ![ "number", "string" ].includes(typeof height)) throw new Error("Height must be a string or number if specified");
  if (void 0 !== filter && (!filter || "object" != typeof filter)) throw new Error("Filter must be an object if specified");
  if (void 0 !== autorefresh && "number" != typeof autorefresh) throw new Error("Autorefresh interval must be an object if specified");
  return {
    baseUrl: baseUrl,
    chartId: chartId,
    filter: filter,
    autorefresh: autorefresh,
    width: width,
    height: height
  };
}, embedChart = async (unsafeOptions, container) => {
  if (!(container instanceof HTMLElement)) throw new Error("Target container must be a HTML element");
  const options = getChartOptions(unsafeOptions);
  for (;container.firstChild; ) container.removeChild(container.firstChild);
  const chartUrl = getChartUrl(options), embedRoot = createElement("div", {
    style: {
      position: "relative",
      overflow: "hidden",
      width: parseCSSMeasurement(options.width) || "100%",
      height: parseCSSMeasurement(options.height) || "100%"
    }
  }), host = chatty.Chatty.createHost(chartUrl).withSandboxAttribute("allow-scripts").withSandboxAttribute("allow-same-origin").appendTo(embedRoot).build();
  Object.assign(host.iframe.style, {
    position: "absolute",
    top: 0,
    left: 0,
    width: "100%",
    height: "100%",
    border: 0
  }), container.appendChild(embedRoot);
  const connection = await host.connect();
  return new EmbeddedChart(embedRoot, host.iframe, connection, options);
};

var index = {
  embedChart: embedChart
};

exports.EmbeddedChart = EmbeddedChart, exports.default = index, exports.embedChart = embedChart;
